Unit Unit1;

{$mode objfpc}{$H+}

Interface

Uses
  Classes,
  SysUtils,
  Forms,
  Controls,
  Graphics,
  Dialogs,
  ExtCtrls,
  StdCtrls;

Type

  { Tform1 }

  Tform1 = Class(TForm)
    buttonBlowFishTest: TButton;
    Memo1: Tmemo;
    Panel1: Tpanel;
    Panel2: Tpanel;
    Panel3: Tpanel;
    Panel4: Tpanel;
    Procedure buttonBlowFishTestClick(Sender: TObject);
  Private

  Public

  End;

Var
  Form1: Tform1;

Implementation

Uses
  BlowFish,
  ClpSecureRandom,
  ClpISecureRandom,
  hlpHashFactory,
  hlpIHash,
  hlpIHashResult;


{$R *.frm}

{  Uses BlowFish; }
Function BlowFishEncrypt(Const AClearStream: TStream; Const ABlowStream: TStream; Const APassword: String; Const ABufferSize: DWord): Boolean;
Var
  ABlowFish: TBlowFishEncryptStream;
  ABuffer: TBytes;
  ABytesRead: Integer;
Begin
  Result := True;

  AClearStream.Position := 0;
  ABlowStream.Position := 0;
  Try
    ABlowFish := TBlowFishEncryptStream.Create(APassword, ABlowStream);
    Try
      SetLength(ABuffer, ABufferSize);
      Try
        FillChar(ABuffer[0], ABufferSize, 0);
        Repeat
          // Read clear data from clear stream
          ABytesRead := AClearStream.Read(ABuffer, ABufferSize);
          // Write encrypted data to BlowFish stream
          ABlowFish.Write(ABuffer, ABytesRead);
        Until (ABytesRead < ABufferSize);
      Finally
        FillChar(ABuffer[0], Length(ABuffer), 0);
        SetLength(ABuffer, 0);
      End;
    Finally
      FreeAndNil(ABlowFish);
    End;
  Except
    Result := False;
  End;
End;


{  Uses BlowFish; }
Function BlowFishDecrypt(Const ABlowStream: TStream; Const AClearStream: TStream; Const APassword: String; Const ABufferSize: DWord): Boolean;
Var
  ABlowFish: TBlowFishDeCryptStream;
  ABuffer: TBytes;
  ABytesRead: Integer;
Begin
  Result := True;

  ABlowStream.Position := 0;
  AClearStream.Position := 0;
  Try
    ABlowFish := TBlowFishDeCryptStream.Create(APassword, ABlowStream);
    Try
      SetLength(ABuffer, ABufferSize);
      Try
        FillChar(ABuffer[0], ABufferSize, 0);
        Repeat
          // Read encrypted data from BlowFish stream
          ABytesRead := ABlowFish.Read(ABuffer, ABufferSize);
          // write clear data to clear stream
          AClearStream.Write(ABuffer, ABytesRead);
        Until (ABytesRead < ABufferSize);
      Finally
        FillChar(ABuffer[0], Length(ABuffer), 0);
        SetLength(ABuffer, 0);
      End;
    Finally
      FreeAndNil(ABlowFish);
    End;
    // AClearStream.Size := AClearStream.Size - 2;
  Except
    Result := False;
  End;
End;

Procedure FillStream(Const AStream: TStream; Const ASize: Int64);
Var
  ARandom: ISecureRandom;
  ABytes: TBytes;
Begin
  SetLength(ABytes, ASize);
  Try
    ARandom := TSecureRandom.Create;
    Try
      ARandom.NextBytes(ABytes);
      AStream.WriteBuffer(ABytes, Length(ABytes));
    Finally
      ARandom := Nil;
    End;
  Finally
    SetLength(ABytes, 0);
  End;
End;

{ Tform1 }

Procedure Tform1.buttonBlowFishTestClick(Sender: TObject);
Var
  AInStream,
  ABlowStream,
  AOutStream: TMemoryStream;

  AInSize,
  ABlowSize,
  AOutSize: Int64;

  I: Integer;
  AInHash, AOutHash: String;
  AMatch : Boolean;
Begin
  memo1.Lines.Clear;

  For I := 0 To 127 Do
  Begin
    AInStream := TMemoryStream.Create;
    ABlowStream := TMemoryStream.Create;
    AOutStream := TMemoryStream.Create;
    Try
      FillStream(AInStream, $FFF + I);
      BlowFishEncrypt(AInStream, ABlowStream, 'Password', $FF);
      BlowFishDecrypt(ABlowStream, AOutStream, 'Password', $FF);

      AInSize := AInStream.Size;
      ABlowSize := ABlowStream.Size;
      AOutSize := AOutStream.Size;

      AInStream.Seek(0, soBeginning);
      AInHash := THashFactory.TCrypto.CreateMD5.ComputeStream(AInStream).ToString;

      AOutStream.Seek(0, soBeginning);
      AOutHash := THashFactory.TCrypto.CreateMD5.ComputeStream(AOutStream).ToString;

      AMatch := (AInHash = AOutHash);

      memo1.Lines.Add('%3d.  InSize:%4d, BlowSize:%4d, OutSize:%4d, InHash: %s, OutHash: %s - Match: %s',
        [I, AInSize, ABlowSize, AOutSize, AInHash, AOutHash, BoolToStr(AMatch, True)]);
    Finally
      FreeAndNil(AOutStream);
      FreeAndNil(ABlowStream);
      FreeAndNil(AInStream);
    End;
  End;
End;



End.
